evoq_event_handler_sup (evoq v1.14.1)

Supervisor for event handler workers.

Manages event handlers that subscribe to event types (not streams). Each handler declares interest in specific event types via the interested_in/0 callback.

Summary

Functions

Start an event handler worker with default config.

Start an event handler worker with config.

Start the event handler supervisor.

Stop an event handler worker.

Functions

start_handler(HandlerModule)

-spec start_handler(atom()) -> {ok, pid()} | {error, term()}.

Start an event handler worker with default config.

start_handler(HandlerModule, Config)

-spec start_handler(atom(), map()) -> {ok, pid()} | {error, term()}.

Start an event handler worker with config.

start_link()

-spec start_link() -> {ok, pid()} | {error, term()}.

Start the event handler supervisor.

stop_handler(HandlerModule)

-spec stop_handler(atom()) -> ok | {error, term()}.

Stop an event handler worker.