evoq_aggregate_registry (evoq v1.14.1)
View Sourcepg-based registry for aggregate processes.
Uses OTP pg module for process registration and lookup. Aggregates register with a group key of {aggregate, AggregateId}.
Summary
Functions
get_or_start(AggregateModule, AggregateId)
deprecated
Get an existing aggregate or start a new one (uses env store_id).
Get an existing aggregate or start a new one with explicit store_id.
Lookup an aggregate by ID.
Register an aggregate process.
Start the aggregate registry.
Unregister an aggregate process.
Functions
This function is deprecated. Use get_or_start/3 with explicit store_id instead..
Get an existing aggregate or start a new one (uses env store_id).
Get an existing aggregate or start a new one with explicit store_id.
Lookup an aggregate by ID.
Register an aggregate process.
Start the aggregate registry.
-spec unregister(binary()) -> ok.
Unregister an aggregate process.