evoq_aggregates_sup (evoq v1.9.2)
View SourceSupervisor for aggregate processes.
Implements partitioned supervision for aggregates. Uses 4 partition supervisors to distribute load and prevent single-supervisor bottlenecks.
Aggregates are distributed across partitions using phash2(StreamId, 4).
Summary
Functions
Get an existing aggregate process.
Calculate partition for an aggregate ID.
Start an aggregate process in the appropriate partition (uses env store_id).
Start an aggregate process in the appropriate partition with explicit store_id.
Start the aggregates supervisor.
Functions
Get an existing aggregate process.
-spec partition_for(binary()) -> 1..4.
Calculate partition for an aggregate ID.
Start an aggregate process in the appropriate partition (uses env store_id).
Start an aggregate process in the appropriate partition with explicit store_id.
Start the aggregates supervisor.